/*
 *   Memory File Picker
 *   Copyright (C) 2019, 2020 Christian Pauly
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

library memoryfilepicker;

import 'dart:typed_data';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/foundation.dart';
import 'package:image_picker/image_picker.dart';
import 'package:universal_html/prefer_universal/html.dart' as html;
import 'dart:async';

abstract class CustomFilePicker {
  final ImagePicker _imagePicker = ImagePicker();

  /// Returns a `MemoryFile` object from the selected file path.
  /// on Web [type], [allowedExtensions] and [onFileLoading] will
  /// be ignored.
  static Future<MemoryFile> getFile({
    FileType type = FileType.any,
    List<String> allowedExtensions,
    Function(dynamic) onFileLoading,
  }) async {
    if (!kIsWeb) {
      final file = await FilePicker.getFile(
        type: type,
        allowedExtensions: allowedExtensions,
        onFileLoading: onFileLoading,
      );
      if (file == null) return null;
      return MemoryFile()
        ..bytes = await file.readAsBytes()
        ..path = file.path;
    }
    return _pickWebFile();
  }

  /// Returns a `MemoryFile` object from the selected image. On web
  /// [maxWidth], [maxHeight], [imageQuality], [source] and [preferredCameraDevice]
  /// will be ignored.
  Future<MemoryFile> getImage({
    @required ImageSource source,
    double maxWidth,
    double maxHeight,
    int imageQuality,
    CameraDevice preferredCameraDevice = CameraDevice.rear,
  }) async {
    if (!kIsWeb) {
      final file = await _imagePicker.getImage(
        source: source,
        maxHeight: maxHeight,
        maxWidth: maxWidth,
        imageQuality: imageQuality,
        preferredCameraDevice: preferredCameraDevice,
      );
      if (file == null) return null;
      return MemoryFile()
        ..bytes = await file.readAsBytes()
        ..path = file.path;
    }
    return _pickWebFile(imagePicker: true);
  }

  static Future<MemoryFile> _pickWebFile({bool imagePicker = false}) async {
    final html.InputElement input = html.document.createElement('input');
    input..type = 'file';
    if (imagePicker) input..accept = 'image/*';
    input.click();
    await input.onChange.first;
    var files2 = input.files;
    final files = files2;
    final dynamic reader = html.FileReader();
    reader.readAsArrayBuffer(files[0]);
    await reader.onLoad.first;
    final byteList = reader.result as List<int>;
    return MemoryFile()
      ..bytes = byteList
      ..path = files[0].name;
  }
}

class MemoryFile {
  Uint8List bytes;
  String path;
}
